# -*- coding: utf-8 -*-
"""
@Author: ChenXiaolei
@Date: 2020-05-21 10:31:14
@LastEditTime: 2020-05-21 10:36:41
@LastEditors: ChenXiaolei
@Description: 控制台主文件
"""

from seven_framework.console.base_console import *


def main():
    pass


if __name__ == "__main__":
    logger_info.info("application begin")
    main()
